Module bulk_vec_extender

Module bulk_vec_extender 

BulkVecExtender is a simple utility trait that allows you to bulk extend a Vec and return a &mut [T] slice that you can write to - much faster than individual push() calls, which has to check for both bounds and capacity.

Structs

BulkExtendGuard
RAII guard for bulk Vec extension. Automatically finalizes on drop.

Traits

BulkVecExtender
BulkVecExtender is a simple utility trait that allows you to bulk extend a Vec and return a &mut [T] slice that you can write to - much faster than individual push() calls, which has to check for both bounds and capacity.
SliceU8SIMDExtender
Utility trait to help write u8 SIMD vectors into a mutable slice